PositionABI = "(int64,uint64,int64,uint32,bool)"
SpotBalanceABI = "(uint64,uint64,uint64)"
UserVaultEquityABI = "(uint64,uint64)"
DelegationsABI = "(address,uint64,uint64)[]"
DelegatorSummaryABI = "(uint64,uint64,uint64,uint64)"
PerpAssetInfoABI = "(string,uint32,uint8,uint8,bool)"
SpotInfoABI = "(string,uint64[2])"
TokenInfoABI = "(string,uint64[],uint64,address,address,uint8,uint8,int8)"
TokenSupplyABI = "(uint64,uint64,uint64,uint64,(address,uint64)[])"
BboABI = "(uint64,uint64)"
AccountMarginSummaryABI = "(int64,uint64,uint64,int64)"
